import json

from dasl_api import (
    CoreV1RuleSpec,
    CommonV1ObjectMeta,
    CoreV1Rule,
    CoreV1Schedule,
    CoreV1RuleSpecInput,
    CoreV1RuleSpecOutput,
    CoreV1RuleSpecMetadata,
    CoreV1RuleObservable,
    CoreV1RuleSpecCollate,
    CoreV1RuleList,
)

from dasl_client.core.base import BaseMixin
from typing import List, Optional
import dasl_api as openapi_client

from dasl_client.errors.errors import NotFoundError, handle_errors


class RuleMixin(BaseMixin):
    @handle_errors
    def get_rule(self, name: str) -> CoreV1Rule:
        """
        Get a previously created rule by name.

        :param name: The name of the rule to fetch
        :return: a CoreV1Rule object containing the spec and status of the rule.
        :raises: NotFoundError if the rule does not exist
        """
        auth = self.auth.client()
        workspace = self.auth.workspace()
        return openapi_client.CoreV1Api(auth).core_v1_get_rule(
            workspace, name
        )

    @handle_errors
    def delete_rule(self, name: str) -> None:
        """
        Delete a rule, request all deployed resource be removed as well. Note,
        this call is an instruction to ASL to delete the rule, it does not
        block until the rule has been deleted.

        :param name: The name of the rule to delete
        :raises: NotFoundError if the rule does not exist
        """
        auth = self.auth.client()
        workspace = self.auth.workspace()
        openapi_client.CoreV1Api(auth).core_v1_delete_rule(workspace, name)

    @handle_errors
    def list_rules(
        self, cursor: Optional[str] = None, limit: Optional[int] = None
    ) -> CoreV1RuleList:
        """
        List all rules in the current client.

        :param cursor: An optional identifier generated by the server. This is
                       used to fetch rules following on from prior list
                       request which returned a cursor.
        :param limit:  An optional limit on the number of rules to return.
        :return: A CoreV1RuleList containing the rules returned.
        """
        auth = self.auth.client()
        workspace = self.auth.workspace()
        return openapi_client.CoreV1Api(auth).core_v1_list_rules(
            workspace=workspace, cursor=cursor, limit=limit
        )

    @handle_errors
    def create_rule(
        self,
        name: str,
        schedule: CoreV1Schedule,
        input: CoreV1RuleSpecInput,
        output: CoreV1RuleSpecOutput,
        comment: Optional[str] = None,
        metadata: Optional[CoreV1RuleSpecMetadata] = None,
        observables: Optional[List[CoreV1RuleObservable]] = None,
        collate: Optional[CoreV1RuleSpecCollate] = None,
    ) -> CoreV1Rule:
        """
        Create a rule with the information provided in this request.
        Note, this request is an instruction and will return before the new
        rule has been created. It can take time for the changes to propagate
        and the relevant resources to be deployed.

        :param name: The name of the rule to replace
        :param schedule: The schedule to apply to the rule.
        :param input: The source table(s) anf filters for the rule.
        :param output: The output summary and context to be associated with any notables
                       generated.
        :param comment: An optional comment for the rule.
        :param metadata: Additional context information associated with the rule.
        :param observables: Any observables to when a noteable is identified.
        :param collate: Conditions under which a generate notable should be merged with
                        a prior notable instead of being treated as a separate event.
        :return: The rule object as recorded by the server.
        :raises ConflictError: if a rule with the same name already exists.
        """

        auth = self.auth.client()
        workspace = self.auth.workspace()

        spec = CoreV1RuleSpec(
            schedule=schedule,
            input=input,
            output=output,
            metadata=metadata,
            observables=observables,
            collate=collate,
        )
        meta = CommonV1ObjectMeta(name=name, comment=comment, workspace=workspace)
        request = CoreV1Rule(metadata=meta, spec=spec)
        return openapi_client.CoreV1Api(auth).core_v1_create_rule(
            workspace, request
        )

    @handle_errors
    def replace_rule(
        self,
        name: str,
        schedule: CoreV1Schedule,
        input: CoreV1RuleSpecInput,
        output: CoreV1RuleSpecOutput,
        metadata: Optional[CoreV1RuleSpecMetadata] = None,
        observables: Optional[List[CoreV1RuleObservable]] = None,
        collate: Optional[CoreV1RuleSpecCollate] = None,
    ) -> CoreV1Rule:
        """
        Replace an existing rule with the information provided in this request.
        Note, this request is an instruction and will return before the new
        rule has been created. It can take time for the changes to propagate
        and the relevant resources to be deployed.

        :param name: The name of the rule to replace
        :param schedule: The schedule to apply to the rule.
        :param input: The source table(s) anf filters for the rule.
        :param output: The output summary and context to be associated with any notables
                       generated.
        :param metadata: Additional context information associated with the rule.
        :param observables: Any observables to when a noteable is identified.
        :param collate: Conditions under which a generate notable should be merged with
                        a prior notable instead of being treated as a separate event.
        :return: The rule object as recorded by the server.
        :raises NotFoundError: if no rule with the provided name exists to replace.
        """

        auth = self.auth.client()
        workspace = self.auth.workspace()

        spec = CoreV1RuleSpec(
            schedule=schedule,
            input=input,
            output=output,
            metadata=metadata,
            observables=observables,
            collate=collate,
        )
        meta = CommonV1ObjectMeta(workspace=workspace)
        request = CoreV1Rule(metadata=meta, spec=spec)
        return openapi_client.CoreV1Api(auth).core_v1_replace_rule(
            workspace, name, request
        )



from dasl_api import (
    DbuiV1TransformRequest,
    DbuiV1TransformRequestInput,
    CoreV1DataSourceAutoloaderSpec,
    DbuiV1TransformRequestTransformsInner,
    DbuiV1TransformResponse,
)

from dasl_client.core.base import BaseMixin
from typing import List, Optional
import dasl_api as openapi_client

from dasl_client.errors.errors import handle_errors
from pydantic import Field


class AdhocTransform(BaseMixin):
    @handle_errors
    def adhoc_transform(
        self,
        warehouse: str,
        preset: str,
        transforms: List[DbuiV1TransformRequestTransformsInner],
        input: Optional[DbuiV1TransformRequestInput] = None,
        autoloader_input: Optional[CoreV1DataSourceAutoloaderSpec] = None,
    ) -> DbuiV1TransformResponse:
        """
        Run a sequence of ADHOC transforms against a SQL warehouse to mimic the operations performed by a datasource.

        :param warehouse: The warehouse ID to run the transforms against.
        :param transforms: The sequence to transforms to run.
        :param input: The source of data to use.
        :param autoloader_input: The autoload specifications.
        :return: a DbuiV1TransformResponse object containing the results after running the transforms.
        :raises: NotFoundError if the rule does not exist
        """
        auth = self.auth.client()
        workspace = self.auth.workspace()
        if autoloader_input is None:
            autoloader_input = Field(default=None, alias="autoloaderInput")
        request = DbuiV1TransformRequest(
            input=input,
            autoloader_input=autoloader_input,
            use_preset=preset,
            transforms=transforms,
        )
        return openapi_client.DbuiV1Api(auth).dbui_v1_transform(
            workspace, warehouse, request
        )



from typing import Optional

from dasl_api import (
    api,
    WorkspaceV1AdminConfigSpec,
    WorkspaceV1CreateWorkspaceRequest,
    WorkspaceV1AdminConfigSpecAuth,
    WorkspaceV1AdminConfigSpecAuthAppClientId,
    WorkspaceV1AdminConfigSpecAuthServicePrincipal,
)

from dasl_client.auth.auth import (
    ServiceAccountKeyAuth,
    DatabricksSecretAuth,
    Authorization,
)
from dasl_client.conn.conn import get_base_conn
from dasl_client.core.config import AdminConfig, ConfigMixin
from dasl_client.core.datasource import DatasourceMixin
from dasl_client.dbui.transform import AdhocTransform
from dasl_client.core.rule import RuleMixin
from dasl_client.core.transform import TransformMixin
from dasl_client.errors.errors import ConflictError, handle_errors


class Client(ConfigMixin, RuleMixin, DatasourceMixin, TransformMixin, AdhocTransform):
    """
    An Antimatter Security Lakehouse client conn.
    """

    def __init__(self, name: str, email: str, auth: Authorization):
        """
        Initialise a new client.
        """
        self.name = name
        self.email = email
        self.auth = auth

    @staticmethod
    @handle_errors
    def new_client(
        name: str,
        email: str,
        config: AdminConfig,
    ) -> "Client":
        """
        Create a new client with the provided name and email and return the associate conn.

        :param self:
        :param name: The proposed name of the client.
        :param email: The email to as the admin contact.
         :param config: admin config settings for access to Databricks.
        :return: A client conn.
        :raises:
            ConflictError - If a client with the given name already exists.
            urllib3.exceptions.MaxRetryError - If we failed to establish a connection to the API.
            Exception - Unknown general exception.
        """
        req = WorkspaceV1CreateWorkspaceRequest(
            admin_user=email,
            workspace_name=name,
            admin_config=WorkspaceV1AdminConfigSpec(
                auth=WorkspaceV1AdminConfigSpecAuth(
                    host=config.host,
                    app_client_id=WorkspaceV1AdminConfigSpecAuthAppClientId(
                        client_id=config.client_id,
                    ),
                    service_principal=WorkspaceV1AdminConfigSpecAuthServicePrincipal(
                        client_id=config.service_principal_id,
                        secret=config.service_principal_secret,
                    ),
                ),
            ),
        )
        api_client = get_base_conn()
        client = api.WorkspaceV1Api(api_client=api_client)

        rsp = client.workspace_v1_create_workspace(req)
        key = rsp.admin_service_account.apikey
        return Client(name, email, ServiceAccountKeyAuth(name, key))

    @staticmethod
    @handle_errors
    def get_client(name: str, host: str, service_account_token: Optional[str] = None) -> "Client":
        """
        Try build a conn from an existing client, using the databricks
        context token as auth.
        :param name: The name of the client to connect to.
        :param host: An option URL for the DASL server. Will use the default if
                     not supplied.
        :param service_account_token: Optional service account token. if
               provided we will attempt to make a client form this.
        :return:
        """
        if service_account_token is not None:
            return Client(name, "", ServiceAccountKeyAuth(name, service_account_token))
        return Client(name, "", DatabricksSecretAuth(name, host))

    @staticmethod
    @handle_errors
    def new_or_existing_client(
        name: str,
        email: str,
        host: str,
        config: AdminConfig,
    ) -> "Client":
        """
        Create a new client with the provided name and email and return
        the associate conn, or if the workspace already exists, connect
        to it using the provided name and host.

        :param self:
        :param name: The proposed name of the client.
        :param email: The email to as the admin contact.
        :param host: An option URL for the DASL server. Will use the default if
                     not supplied.
        :param config: admin config settings. If creating a new workspace,
            these will be sent with the request. If the workspace already
            exists, the workspace admin config will be updated.
        :return: A client conn.
        :raises:
            urllib3.exceptions.MaxRetryError - If we failed to establish a connection to the API.
            Exception - Unknown general exception.
        """
        try:
            return Client.new_client(name, email, config)
        except ConflictError:
            result = Client.get_client(name, host)
            result.put_admin_config(config)
            return result



import random
import string
import time
import unittest

from dasl_api import (
    WorkspaceV1WorkspaceConfigSpecObservables,
    WorkspaceV1WorkspaceConfigSpecObservablesKindsInner,
    WorkspaceV1WorkspaceConfigSpecDefaultConfig,
    WorkspaceV1DefaultConfig,
    CoreV1Schedule,
    CoreV1RuleSpecInput,
    CoreV1RuleSpecOutput,
    CoreV1RuleSpecInputStream,
    CoreV1RuleSpecInputStreamTablesInner,
    CoreV1RuleObservable,
    CoreV1RuleObservableRisk,
    CoreV1DataSourceSpecSilver,
    CoreV1DataSourceSpecGold,
    CoreV1DataSourceSpecSilverPreTransform,
    CoreV1DataSourceSpecBronze,
)

from dasl_client.client.client import Client


class MyTestCase(unittest.TestCase):
    @staticmethod
    def create_workspace(self, email) -> Client:
        workspace_name = "".join(random.choices(string.ascii_lowercase, k=16))
        workspace = Client.new_client(workspace_name, email)

        workspace.put_admin_config(
            host="https://dbc-958e1220-59fc.cloud.databricks.com",
            client_id="22853b93-68ba-4ae2-8e41-976417f501dd",
            service_principal_id="",
            service_principal_secret="",
        )

        workspace.put_config(
            system_tables_catalog_name="databricks_dev",
            system_tables_schema="asl_system_tables",
            detection_categories=["example"],
            observables=WorkspaceV1WorkspaceConfigSpecObservables(
                kinds=[
                    WorkspaceV1WorkspaceConfigSpecObservablesKindsInner(
                        name="ip_source",
                        sql_type="string",
                    ),
                    WorkspaceV1WorkspaceConfigSpecObservablesKindsInner(
                        name="ip_destination",
                        sql_type="string",
                    ),
                ],
                relationships=["tenant", "tenant"],
            ),
            dasl_storage_path="/Volumes/databricks_dev/default/test-itg",
            default_config=WorkspaceV1WorkspaceConfigSpecDefaultConfig(
                datasources=WorkspaceV1DefaultConfig(
                    notebook_location="/Workspace/Shared/datasources/notebooks",
                    bronze_schema="bronze",
                    silver_schema="silver",
                    gold_schema="gold",
                    catalog_name="default",
                ),
                transforms=WorkspaceV1DefaultConfig(
                    notebook_location="/Workspace/Shared/transforms/notebooks",
                    bronze_schema="bronze",
                    silver_schema="silver",
                    gold_schema="gold",
                    catalog_name="default",
                ),
                rules=WorkspaceV1DefaultConfig(
                    notebook_location="/Workspace/Shared/rules/notebooks",
                    bronze_schema="bronze",
                    silver_schema="silver",
                    gold_schema="gold",
                    catalog_name="default",
                ),
                var_global=WorkspaceV1DefaultConfig(
                    bronze_schema="bronze",
                    silver_schema="silver",
                    gold_schema="gold",
                    catalog_name="default",
                ),
            ),
        )
        return workspace

    def test_create_rule(self):
        workspace = self.create_workspace(self, "sean@antimatter.io")
        rule_name_1 = f"{workspace.name}-rule-1"

        workspace.create_rule(
            name=rule_name_1,
            schedule=CoreV1Schedule(at_least_every="2h"),
            input=CoreV1RuleSpecInput(
                stream=CoreV1RuleSpecInputStream(
                    filter="ip_count > 999",
                    tables=[
                        CoreV1RuleSpecInputStreamTablesInner(
                            name="databricks_dev.antimatter_meta.test_ip_summaries",
                            alias="ip",
                        ),
                        CoreV1RuleSpecInputStreamTablesInner(
                            name="databricks_dev.antimatter_meta.server_load",
                            join_expr="ip.region_id = server.region_id",
                            join_type="inner",
                            alias="server",
                        ),
                    ],
                )
            ),
            output=CoreV1RuleSpecOutput(
                summary="rule 1: we found {ip_count} unique IPs",
                context={
                    "IP count": "{ip_count}",
                    "Comment is:": "context: user, comment: {comment}",
                },
            ),
            observables=[
                CoreV1RuleObservable(
                    kind="ip_source",
                    value="ip_count",
                    relationship="tenant",
                    risk=CoreV1RuleObservableRisk(
                        impact="ip_count/1000", confidence="80"
                    ),
                ),
                CoreV1RuleObservable(
                    kind="ip_destination",
                    value="ip_count",
                    relationship="tenant",
                    risk=CoreV1RuleObservableRisk(
                        impact="ip_count/1000", confidence="80"
                    ),
                ),
            ],
        )

        while workspace.get_rule(rule_name_1).status is None:
            time.sleep(2)
        self.assertEqual(workspace.get_rule(rule_name_1).status.job_status, "scheduled")
        time.sleep(5)
        workspace.delete_rule(rule_name_1)

    def test_create_datasource(self):
        workspace = self.create_workspace(self, "sean@antimatter.io")
        datasource_name = f"{workspace.name}-datasource"

        s = CoreV1DataSourceSpecSilver(
            pre_transform=CoreV1DataSourceSpecSilverPreTransform(
                use_preset="aws_sec_lake_route53",
            )
        )

        workspace.create_datasource(
            name=datasource_name,
            schedule=CoreV1Schedule(at_least_every="2h"),
            source="aws",
            source_type="route53",
            use_preset="aws",
            bronze=CoreV1DataSourceSpecBronze(bronze_table="aws_route53_data"),
            silver=CoreV1DataSourceSpecSilver(
                pre_transform=CoreV1DataSourceSpecSilverPreTransform(
                    use_preset="aws_sec_lake_route53",
                )
            ),
            gold=CoreV1DataSourceSpecGold(),
        )

        while workspace.get_datasource(datasource_name).status is None:
            time.sleep(2)
        self.assertEqual(
            workspace.get_datasource(datasource_name).status.job_status, "scheduled"
        )
        time.sleep(5)
        workspace.delete_datasource(datasource_name)


if __name__ == "__main__":
    unittest.main()



from typing import Any, Callable, Dict, List, Optional
from pydantic import BaseModel

import dasl_api as openapi_client
from dasl_api import (
    WorkspaceV1WorkspaceConfig,
    WorkspaceV1WorkspaceConfigSpec,
    WorkspaceV1WorkspaceConfigSpecSystemTablesConfig,
    WorkspaceV1ExportConfig,
    WorkspaceV1WorkspaceConfigSpecObservables,
    WorkspaceV1WorkspaceConfigSpecComputeGroupLimitsInner,
    WorkspaceV1WorkspaceConfigSpecDefaultConfig,
    WorkspaceV1WorkspaceConfigSpecManagedRetentionInner,
    WorkspaceV1WorkspaceConfigSpecDetectionRuleMetadata,
    CommonV1ObjectMeta,
    WorkspaceV1AdminConfig,
    WorkspaceV1AdminConfigSpec,
    WorkspaceV1AdminConfigSpecAuth,
    WorkspaceV1AdminConfigSpecAuthServicePrincipal,
    WorkspaceV1AdminConfigSpecAuthAppClientId,
)

from dasl_client.core.base import BaseMixin
from dasl_client.errors.errors import handle_errors


class AdminConfig(BaseModel):
    host: str
    client_id: str
    service_principal_id: str
    service_principal_secret: str


class ConfigMixin(BaseMixin):
    """
    Client mixin for CRUD operations on config
    """

    @handle_errors
    def get_config(self) -> WorkspaceV1WorkspaceConfig:
        """
        Get the current configuration for the client.

        :return: WorkspaceV1WorkspaceConfig object
        :exception: NotFoundError if the client does not have a configuration.
        """
        auth = self.auth.client()
        workspace = self.auth.workspace()
        client = openapi_client.WorkspaceV1Api(auth)
        return client.workspace_v1_get_config(workspace)

    @handle_errors
    def put_admin_config(self, config: AdminConfig) -> WorkspaceV1AdminConfig:
        """
        Update the admin configuration for the client.

        :param host: The url of the databricks client instance.
        :param service_principal_id: The ID of the databricks service principal.
        :param service_principal_secret: The  databricks service principal secret.
        :param client_id: The databricks conn ID.
        :return: The WorkspaceV1AdminConfig after applying the configuration
        """
        api_auth = self.auth.client()
        workspace = self.auth.workspace()
        client = openapi_client.WorkspaceV1Api(api_auth)

        auth = WorkspaceV1AdminConfigSpecAuth(
            host=config.host,
            service_principal=WorkspaceV1AdminConfigSpecAuthServicePrincipal(
                client_id=config.service_principal_id,
                secret=config.service_principal_secret,
            ),
            app_client_id=WorkspaceV1AdminConfigSpecAuthAppClientId(
                client_id=config.client_id
            ),
        )

        req = WorkspaceV1AdminConfig(
            api_version="v1",
            kind="WorkspaceConfig",
            spec=WorkspaceV1AdminConfigSpec(auth=auth),
        )

        return client.workspace_v1_put_admin_config(workspace, req)

    @handle_errors
    def put_config(
        self,
        system_tables_catalog_name: str = "",
        system_tables_schema: str = "",
        default_sql_warehouse: Optional[str] = None,
        detection_categories: Optional[List[str]] = None,
        notable_export: Optional[WorkspaceV1ExportConfig] = None,
        operational_alert_export: Optional[WorkspaceV1ExportConfig] = None,
        observables: Optional[WorkspaceV1WorkspaceConfigSpecObservables] = None,
        compute_group_limits: Optional[
            List[WorkspaceV1WorkspaceConfigSpecComputeGroupLimitsInner]
        ] = None,
        dasl_storage_path: Optional[str] = None,
        default_config: Optional[
            WorkspaceV1WorkspaceConfigSpecDefaultConfig
        ] = None,
        managed_retention: Optional[
            List[WorkspaceV1WorkspaceConfigSpecManagedRetentionInner]
        ] = None,
    ) -> None:
        """
        Update the configuration for the client.

        :param system_tables_catalog_name: The catalog in Databricks under which system tables should be stored.
        :param system_tables_schema: The catalog schema in Databricks under which system tables should be stored.
        :param default_sql_warehouse: The default SQL warehouse for DASL to use.
        :param detection_categories:
        :param notable_export: Optional parameters to enable exporting of notables.
        :param operational_alert_export: Optional parameters to enable exporting of operations alerts.
        :param observables: Observables that can be collected when generating rules.
        :param compute_group_limits: Limit to apply to compute groups.
        :param dasl_storage_path: The root path under which DASL resources will be stored.
        :param default_config: the default configuration for DASL resources.
        :param managed_retention: Retention characteristics to apply to generated data.
        :return: The config object as recorded by the server.
        """
        spec = WorkspaceV1WorkspaceConfigSpec(
            system_tables_config=WorkspaceV1WorkspaceConfigSpecSystemTablesConfig(
                catalog_name=system_tables_catalog_name,
                var_schema=system_tables_schema,
            ),
            default_sql_warehouse=default_sql_warehouse,
            notable_export=notable_export,
            operational_alert_export=operational_alert_export,
            observables=observables,
            compute_group_limits=compute_group_limits,
            dasl_storage_path=dasl_storage_path,
            default_config=default_config,
            managed_retention=managed_retention,
        )

        if detection_categories is not None:
            spec.detection_rule_metadata = (
                WorkspaceV1WorkspaceConfigSpecDetectionRuleMetadata(
                    detection_categories=detection_categories
                )
            )

        auth = self.auth.client()
        workspace = self.auth.workspace()

        metadata = CommonV1ObjectMeta(workspace=workspace, name="config")
        request = WorkspaceV1WorkspaceConfig(
            api_version="v1", kind="WorkspaceConfig", metadata=metadata, spec=spec
        )
        client = openapi_client.WorkspaceV1Api(auth)
        client.workspace_v1_put_config(workspace, request)



from dasl_client.auth.auth import Authorization


class BaseMixin:
    def __init__(self, auth: Authorization, **kwargs):
        try:
            super().__init__(auth=auth, **kwargs)
        except TypeError:
            super().__init__()  # If this is last mixin, super() will be object()
        self.auth = auth



from dasl_api import (
    CoreV1DataSource,
    CoreV1DataSourceList,
    CommonV1ObjectMeta,
    CoreV1DataSourceSpec,
    CoreV1Schedule,
    CoreV1DataSourceSpecSilver,
    CoreV1DataSourceSpecGold,
    CoreV1DataSourceAutoloaderSpec,
    CoreV1DataSourceSpecBronze,
    CoreV1DataSourceSpecCustom,
)

from dasl_client.core.base import BaseMixin
from typing import List, Optional
import dasl_api as openapi_client

from dasl_client.errors.errors import NotFoundError, handle_errors


class DatasourceMixin(BaseMixin):
    @handle_errors
    def get_datasource(self, name: str) -> CoreV1DataSource:
        """
        Get a previously created datasource by name.

        :param name: The name of the datasource to fetch
        :return: a CoreV1DataSource object containing the spec and status of the datasource.
        :raises: NotFoundError if the datasource does not exist
        """
        auth = self.auth.client()
        workspace = self.auth.workspace()
        return openapi_client.CoreV1Api(auth).core_v1_get_data_source(
            workspace, name
        )

    @handle_errors
    def delete_datasource(self, name: str) -> None:
        """
        Delete a datasource, request all deployed resource be removed as well. Note,
        this call is an instruction to ASL to delete the datasource, it does not
        block until the datasource has been deleted.

        :param name: The name of the datasource to delete
        :raises: NotFoundError if the datasource does not exist
        """
        auth = self.auth.client()
        workspace = self.auth.workspace()
        openapi_client.CoreV1Api(auth).core_v1_delete_data_source(
            workspace, name
        )

    @handle_errors
    def list_datasources(
        self, cursor: Optional[str] = None, limit: Optional[int] = None
    ) -> CoreV1DataSourceList:
        """
        List all datasources in the current client.

        :param cursor: An optional identifier generated by the server. This is
                       used to fetch datasources following on from prior list
                       request which returned a cursor.
        :param limit:  An optional limit on the number of datasources to return.
        :return: A CoreV1DataSourceList containing the datasources returned.
        """
        auth = self.auth.client()
        workspace = self.auth.workspace()
        return openapi_client.CoreV1Api(auth).core_v1_list_data_sources(
            workspace=workspace, cursor=cursor, limit=limit
        )

    @handle_errors
    def create_datasource(
        self,
        name: str,
        source: str,
        source_type: str,
        use_preset: str,
        schedule: CoreV1Schedule,
        custom_notebook_path: Optional[str] = None,
        bronze: Optional[CoreV1DataSourceSpecBronze] = None,
        silver: Optional[CoreV1DataSourceSpecSilver] = None,
        gold: Optional[CoreV1DataSourceSpecGold] = None,
        autoloader_override: Optional[CoreV1DataSourceAutoloaderSpec] = None,
    ) -> CoreV1DataSource:
        """
        Create a datasource with the information provided in this request.
        Note, this request is an instruction and will return before the new
        datasource has been created. It can take time for the changes to
        propagate and the relevant resources to be deployed.

        :param name: Name of the datasource to create.
        :param source: Name of originator of the data we will import (GCP, AWS, Okta, etc).
        :param source_type: The type of data we are importing from the source (e.g. S3, LB, etc).
        :param use_preset: The name of the preset to use for this data source.
        :param input: Describes what and how to import data into bronze tables.
        :param schedule: The schedule to apply to the datasource.
        :param custom_notebook_path: An optional path to the notebook containing custom
                                     functions for the datasource.
        :param bronze: The bronze data source spec.
        :param silver: The silver data source spec.
        :param gold: The  gold data source spec.
        :param autoloader_override: An optional autoloader override for the datasource.
        :return: The datasource as recorded by the server.
        :raises ConflictError: if the datasource already exists.
        """
        auth = self.auth.client()
        workspace = self.auth.workspace()

        spec = CoreV1DataSourceSpec(
            source=source,
            use_preset=use_preset,
            source_type=source_type,
            autoloader=autoloader_override,
            schedule=schedule,
            custom=CoreV1DataSourceSpecCustom(notebook=custom_notebook_path),
            bronze=bronze,
            silver=silver,
            gold=gold,
        )
        meta = CommonV1ObjectMeta(name=name, workspace=workspace)
        request = CoreV1DataSource(metadata=meta, spec=spec)
        return openapi_client.CoreV1Api(auth).core_v1_create_data_source(
            workspace, request
        )

    @handle_errors
    def replace_datasource(
        self,
        name: str,
        source: str,
        source_type: str,
        use_preset: str,
        schedule: CoreV1Schedule,
        custom_notebook_path: Optional[str] = None,
        bronze: Optional[CoreV1DataSourceSpecBronze] = None,
        silver: Optional[CoreV1DataSourceSpecSilver] = None,
        gold: Optional[CoreV1DataSourceSpecGold] = None,
        autoloader_override: Optional[CoreV1DataSourceAutoloaderSpec] = None,
    ) -> CoreV1DataSource:
        """
        Replace an existing datasource with the information provided in this request.
        Note, this request is an instruction and will return before the new
        datasource has been created. It can take time for the changes to
        propagate and the relevant resources to be deployed.

        :param name: Name of the datasource to create.
        :param source: Name of originator of the data we will import (GCP, AWS, Okta, etc).
        :param source_type: The type of data we are importing from the source (e.g. S3, LB, etc).
        :param use_preset: The name of the preset to use for this data source.
        :param schedule: The schedule to apply to the datasource.
        :param custom_notebook_path: An optional path to the notebook containing custom
                                     functions for the datasource.
        :param bronze: The bronze data source spec.
        :param silver: The silver data source spec.
        :param gold: The  gold data source spec.
        :param autoloader_override: An optional autoloader override for the datasource.
        :return: The datasource as recorded by the server.
        :raises NotFoundError: if no datasource with the provided name exists to replace.
        """
        auth = self.auth.client()
        workspace = self.auth.workspace()

        spec = CoreV1DataSourceSpec(
            source=source,
            use_preset=use_preset,
            source_type=source_type,
            autoloader=autoloader_override,
            schedule=schedule,
            custom=CoreV1DataSourceSpecCustom(notebook=custom_notebook_path),
            bronze=bronze,
            silver=silver,
            gold=gold,
        )
        meta = CommonV1ObjectMeta(name=name, workspace=workspace)
        request = CoreV1DataSource(metadata=meta, spec=spec)

        return openapi_client.CoreV1Api(auth).core_v1_replace_data_source(
            workspace, name, request
        )



from dasl_api import (
    CommonV1ObjectMeta,
    CoreV1Transform,
    CoreV1TransformList,
    CoreV1TransformSpec,
    CoreV1TransformSpecStream,
)

from dasl_client.core.base import BaseMixin
from typing import Optional
import dasl_api as openapi_client

from dasl_client.errors.errors import handle_errors


class TransformMixin(BaseMixin):
    @handle_errors
    def get_transform(self, name: str) -> CoreV1Transform:
        """
        Fetch the transform associated with the supplied name.

        :param name: The name of the transform to fetch
        :return: The transform object
        :raises TransformNotFound: if the transform was not found
        """
        auth = self.auth.client()
        workspace = self.auth.workspace()
        return openapi_client.CoreV1Api(auth).core_v1_get_transform(
            workspace, name
        )

    @handle_errors
    def delete_transform(self, name: str) -> None:
        """
        Delete the transform associated with the supplied name.

        :param name: Name of the transform to delete
        :return:
        :raises TransformNotFound: if the transform was not found
        """
        auth = self.auth.client()
        workspace = self.auth.workspace()
        openapi_client.CoreV1Api(auth).core_v1_delete_transform(
            workspace, name
        )

    @handle_errors
    def list_transforms(
        self, cursor: Optional[str] = None, limit: Optional[int] = None
    ) -> CoreV1TransformList:
        """
        List all transforms in the current client.

        :param cursor: An optional identifier generated by the server. This is
                       used to fetch transforms following on from prior list
                       request which returned a cursor.
        :param limit:  An optional limit on the number of transforms to return.
        :return: A CoreV1RuleList containing the transforms returned.
        """
        auth = self.auth.client()
        workspace = self.auth.workspace()
        return openapi_client.CoreV1Api(auth).core_v1_list_transforms(
            workspace=workspace, cursor=cursor, limit=limit
        )

    @handle_errors
    def create_transform(
        self,
        name: str,
        schedule: Optional[str] = None,
        output_table: Optional[str] = None,
        stream: Optional[CoreV1TransformSpecStream] = None,
    ) -> CoreV1Transform:
        """
        Create a new transform with the supplied name. Note, this request is an
        instruction and will return before the new rule has been created. It
        can take time for the changes to propagate and the relevant resources
        to be deployed.

        :param name: name of the transform
        :param schedule: The schedule associated with the transform
        :param output_table: The name of the destination table
        :param stream: The data source to apply the transform to
        :return: The transform object as recorded by the server.
        :raises ConflictError: if a transform with the same name already exists.
        """
        auth = self.auth.client()
        workspace = self.auth.workspace()

        spec = CoreV1TransformSpec(
            schedule=schedule,
            output_table=output_table,
            stream=stream,
        )
        meta = CommonV1ObjectMeta(name=name, workspace=workspace)
        request = CoreV1Transform(metadata=meta, spec=spec)
        return openapi_client.CoreV1Api(auth).core_v1_create_transform(
            workspace, request
        )

    @handle_errors
    def replace_transform(
        self,
        name: str,
        schedule: Optional[str] = None,
        output_table: Optional[str] = None,
        stream: Optional[CoreV1TransformSpecStream] = None,
    ) -> CoreV1Transform:
        """
        Replace a transform associated with the supplied name. Note, this
        request is an instruction and will return before the new rule has been
        created. It can take time for the changes to propagate and the relevant
        resources to be deployed.

        :param name: name of the transform
        :param schedule: The schedule associated with the transform
        :param output_table: The name of the destination table
        :param stream: The data source to apply the transform to
        :return: The transform object as recorded by the server.
        :raises NotFoundError: If no transform with the provided name exists to
                               replace.
        """
        auth = self.auth.client()
        workspace = self.auth.workspace()

        spec = CoreV1TransformSpec(
            schedule=schedule,
            output_table=output_table,
            stream=stream,
        )
        meta = CommonV1ObjectMeta(name=name, workspace=workspace)
        request = CoreV1Transform(metadata=meta, spec=spec)
        return openapi_client.CoreV1Api(auth).core_v1_replace_transform(
            workspace, name, request
        )



from importlib.metadata import version


def get_user_agent() -> str:
    """
    A helper function defining the user agent for requests originating from
    the ASL python conn library. We include the version of the API that the
    conn was built off.
    :return: A user-agent string.
    """
    return f"asl/python-conn/{version('dasl_api')}"
